#!/bin/ksh

function print_help {
  echo "Usage:"
  echo ""
  echo "run_class_unload_performance_tests [-jvm=/path/to/jvm/executable] [-n=number_of_classes] [-h]"
  echo
  echo "    -jvm=/path/to/jvm/executable                  The absolute path to the JVM executable, which will be used for the performance test."
  echo "    --jvm_executable=/path/to/jvm/executable      If not set, will use default executable used by the shell."
  echo ""
  echo "    -n=number_of_classes                          The number of classes that are loaded before unloading and GC triggers begin."
  echo "    --number_of_classes=number_of_classes         Defaults to 100 000 if not set."
  echo ""
  echo "    -h                                            Print this message."
  echo "    --help"
  exit
}

iterations=10
number_of_classes=100000
java_executable=java

for var in "$@"
do
  if [[ ${var} == "--help" || ${var} == "-h" ]]
  then
    print_help
  fi
  value=`echo ${var} | cut -d'=' -f 2`
  if [[ ${var} == -jvm* || ${var} == --jvm_executable* ]]
  then
    java_executable=${value}
  fi
  if [[ ${var} == -n* || ${var} == --number_of_classes* ]]
  then
    number_of_classes=${value}
  fi
done

#
# Factor to use when comparing elapsed time to the no debug agent case.
# If the elapsed time is higher than the base case times this factor,
# the elapsed time will be printed in red (to indicate higher-than-expected running time).
#
threshold_factor=1.2

RED='\033[0;31m'
GREEN='\033[0;32m'
BOLD='\033[1m'
NS='\033[0m'

function print_elapsed {
  threshold=${1}
  elapsed=${2}
  color=${GREEN}
  is_too_high="$(bc <<< "${threshold} < ${elapsed}")"
  if [ "${is_too_high}" == "1" ]
  then
    color=${RED}
  fi
  printf "${color}${BOLD}Elapsed ${elapsed} s${NS}\n"
  printf "\n"
}

classes_jar_path=`pwd`/classes.jar
port_number=8100



function measure_execution_time {
  printf "${BOLD}Running performance test with ${number_of_classes} classes and JVM:${NS}\n"
  printf "${java_executable}\n"
  ${java_executable} -version
  printf "\n"

  ${java_executable}c -cp . test/TestJDKClassUnloadingPerformance.java

  printf "${BOLD}Running snippet without debug agent.${NS}\n"
  start_time="$(date -u +%s.%N)"
  ${java_executable} -cp . -Xmx2G test.TestJDKClassUnloadingPerformance ${classes_jar_path} ${iterations} ${number_of_classes}
  end_time="$(date -u +%s.%N)"
  elapsed_no_agent="$(bc <<< "${end_time} - ${start_time}")"
  printf "${BOLD}Elapsed ${elapsed_no_agent} s${NS}\n"
  printf "\n"

  elapsed_threshold="$(bc <<< "${elapsed_no_agent} * ${threshold_factor}")"

  printf "${BOLD}Running snippet with debug agent and no debugger attached.${NS}\n"
  start_time="$(date -u +%s.%N)"
  ${java_executable} -cp . -Xmx2G -agentlib:jdwp=transport=dt_socket,server=y,suspend=n,address=${port_number} test.TestJDKClassUnloadingPerformance ${classes_jar_path} ${iterations} ${number_of_classes}
  end_time="$(date -u +%s.%N)"
  elapsed_agent="$(bc <<< "${end_time} - ${start_time}")"
  print_elapsed ${elapsed_threshold} ${elapsed_agent}

  printf "${BOLD}Running snippet with debug agent and jdb attached.${NS}\n"
  start_time="$(date -u +%s.%N)"
  ${java_executable} -cp . -agentlib:jdwp=transport=dt_socket,server=y,suspend=y,address=${port_number} -Xmx2G test.TestJDKClassUnloadingPerformance ${classes_jar_path} ${iterations} ${number_of_classes} &
  jdb -launch -attach ${port_number}&
  wait
  end_time="$(date -u +%s.%N)"
  elapsed_agent_attached="$(bc <<< "${end_time} - ${start_time}")"
  print_elapsed ${elapsed_threshold} ${elapsed_agent_attached}
}

measure_execution_time
